# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# Name :          jsAlign.rb 1.01
# Author :        Jan Sandstrom www.pixero.com
# Date :          15 May 2007

require 'sketchup.rb'


class Sketchup::Selection 
def vertices 
	selverts = []; 
	self.each {|e| 
		selverts.push e.vertices if (e.respond_to?(:vertices)); 
	} 
	return selverts.flatten.uniq; 
end ;  # method vertices
end ;  # class Sketchup::Selection 


class AutofoldWatch < Sketchup::EntitiesObserver
	def onElementAdded(element)
		autofoldEdges = [];
		if element.typename == "Edge" and element.valid?
			autofoldEdges.push(element)
		end#if
		return autofoldEdges;
	end#def
end



def jsAlign 
	model = Sketchup.active_model
	entities = model.active_entities
	sel = model.selection
	
	if sel.empty?
     		UI.beep
     		UI.messagebox("Nothing Selected!")
     		return nil
  	end#if
  	
  	### Get all edges in selection
	
	edges=[]
	for e in sel
  		if e.typename=="Edge" and e.valid?
    			edges.push(e)
  		end#if
	end#for
  	
  	
  	### Dialog

	@axis = "Z" if not @axis
	@base = "Center" if not @base
	@customvalue = 0.mm if not defined? @customvalue  and model.options["UnitsOptions"]["LengthUnit"]>1

	enums = ["X" '|' "Y" '|' "Z", "Max" '|' "Center" '|' "Min" '|' "Custom"]
	prompts = ["Align axis: ", "Align to: ", "Custom value: "]
	values = [@axis, @base, @customvalue]
	results = inputbox (prompts, values, enums, "Align Settings")
	#return nil if not results
	axis, base, customvalue = results
	@axis, @base, @customvalue = axis, base, customvalue
  	
  	### Align
  		
	selverts = Sketchup.active_model.selection.vertices; 
	bb = Geom::BoundingBox.new;
	Sketchup.active_model.selection.each {|e| bb.add(e.bounds); } 
		

	if (@base == "Max")
		basepoint = Geom::Point3d.new (bb.max);
	end#if
	if (@base == "Center")
		basepoint = Geom::Point3d.new (bb.center);
	end#if
	if (@base == "Min")
		basepoint = Geom::Point3d.new (bb.min);
	end#if
	if (@base == "Custom")
		basepoint = Geom::Point3d.new ([customvalue, customvalue, customvalue]);
	end#if
	
	@af = Sketchup.active_model.entities.add_observer(AutofoldWatch.new) ###Observer added

	selverts.each {|v| 
	
		point = Geom::Point3d.new [v.position.x, v.position.y, v.position.z]
		
		if (@axis == "X")
			dist = Geom::Point3d.new [basepoint.x-point.x, 0, 0];
		end#if
		if (@axis == "Y")
			dist = Geom::Point3d.new [0, basepoint.y-point.y, 0];
		end#if
		if (@axis == "Z")
			dist = Geom::Point3d.new [0, 0, basepoint.z-point.z];
		end#if
		
				
		t = Geom::Transformation.new (dist);
		Sketchup.active_model.entities.transform_entities(t, v);

	}
	
	autofoldEdges.erase! ###Erase edges created by autofold
	Sketchup.active_model.entities.remove_observer @af ###Observer removed

end 

############################################################
if( not file_loaded?("jsAlign.rb") )
    UI.menu("Plugins").add_separator
    UI.menu("Plugins").add_item("JS Align") { jsAlign  }
end

file_loaded("jsAlign.rb")
